import jarvis
import re
import aiohttp
import html
from jarvis.config import get_config
from jarvis.utils import build_embed
from jarvis.utils.field import Field
from jarvis.data.dbrand import shipping_array
from discord.ext import commands


class DbrandCog(commands.Cog):
    """
    dbrand functions for J.A.R.V.I.S.

    Mostly support functions. Credit @cpixl for the shipping API
    """

    def __init__(self, bot):
        self.bot = bot
        self.base_url = "https://dbrand.com/"
        self._session = aiohttp.ClientSession()
        self._session.headers.update({"Content-Type": "application/json"})
        self.api_url = get_config().urls["dbrand_shipping"]

    @commands.group(name="db", aliases=["dbrand"], pass_context=True)
    async def _db(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send(
                "Available subcommands: `grip`, `support`, "
                + "`skin`, `prism`, `shipping`"
            )

    @_db.command(name="skin", aliases=["skins", "tape"])
    async def _skin(self, ctx):
        await ctx.send(self.base_url + "shop/skins")

    @_db.command(name="grip")
    async def _grip(self, ctx):
        await ctx.send(self.base_url + "shop/grip/#grip-devices")

    @_db.command(name="support")
    async def _support(self, ctx):
        await ctx.send(self.base_url + "contact")

    @_db.command(name="shipping")
    async def _shipping(self, ctx, *, search: str):
        if not re.match(r"^[A-Z- ]+$", search, re.IGNORECASE):
            await ctx.send("Please use text to search for shipping.")
        dest = search
        api_link = self.api_url + dest
        try:
            data = await self._session.get(api_link)
        except Exception as e:
            print(e)
        fields = None
        if 200 <= data.status < 400:
            data = await data.json()
        else:
            data = None
        if data is not None and data["is_valid"]:
            fields = []
            if data["shipping_available"]:
                fields.append(Field(data["short-name"], data["time-title"]))
                for service in data["shipping_services_available"][1:]:
                    service_data = await self._session.get(
                        self.api_url + dest + "/" + service["url"]
                    )
                    if service_data.status > 400:
                        continue
                    service_data = await service_data.json()
                    fields.append(
                        Field(
                            service_data["short-name"],
                            service_data["time-title"],
                        )
                    )
            country = "-".join(
                x for x in data["country"].split(" ") if x != "the"
            )
            country_urlsafe = country.replace("-", "%20")
            description = f"Click the link above to see shipping time to {data['country']}."
            description += "\n[View all shipping destinations](https://dbrand.com/shipping)"
            description += " | [Check shipping status]"
            description += f"(https://dbrand.com/status#main-content:~:text={country_urlsafe})"
            embed = build_embed(
                title="Shipping to {}".format(data["country"]),
                description=description,
                color="#FFBB00",
                fields=fields,
                url=self.base_url + "shipping/" + country,
            )
            embed.set_thumbnail(url=self.base_url + data["country_flag"][1:])
            embed.set_footer(
                text="dbrand.com",
                icon_url="https://dev.zevaryx.com/db_logo.png",
            )
            await ctx.send(embed=embed)
        else:
            embed = build_embed(
                title="Check Shipping Times",
                description="Country not found.\nYou can [view all shipping "
                + "destinations here](https://dbrand.com/shipping)",
                fields=[],
                url="https://dbrand.com/shipping",
                color="#FFBB00",
            )
            embed.set_thumbnail(url="https://dev.zevaryx.com/db_logo.png")
            embed.set_footer(
                text="dbrand.com",
                icon_url="https://dev.zevaryx.com/db_logo.png",
            )
            await ctx.send(embed=embed)


def setup(bot):
    bot.add_cog(DbrandCog(bot))
